'''
SARx (Sequence Analysis Rx) is a suite of functions to assist in the banal
processing of sequence data and the use tools that analyze it.

Created on Nov 4, 2009


'''

import sys
import re

def openAnything(filename="-", mode='r'):
	"opens either standard input or a readonly file"
	if filename == '-' or filename == "":
		if mode == 'r':
			return sys.stdin
		else:
			return sys.stdout
	else:
		return open(filename, mode)

def tabFormat(*args):
	"Formats almost anything in to a tab separated column string"
	if len(args) == 1:
		if not getattr(args[0], "__iter__", False):
			return str(args[0])
		else:
			return '\t'.join([tabFormat(a) for a in args[0]])
	return '\t'.join([tabFormat(a) for a in args])

def dictZip(one, two):
	"returns a list of (key, value, value) tuples for every key in one or two.  If a key isn't\
	 present in one of the dictionaries, then None is given for its value for that key."
	for key in set(one.keys() + two.keys()):
		valueOne = one[key] if key in one else None
		valueTwo = two[key] if key in two else None
		yield(key, valueOne, valueTwo)

def loadBed(data, all=False):
	"A generator that outputs (chr, start, end, strand) tuples for each line in\
	a BED formatted iterable."
	for line in data:
		fields = line.strip().split()
		if fields[0] == "track":
			continue
		assert(len(fields) >= 3)
		fields[1:3] = int(fields[1]), int(fields[2])
		if all:
			yield tuple(fields)
		elif len(fields) < 6:
			yield tuple(fields[:3])
		else:
			yield (fields[0], fields[1], fields[2], fields[5])

def loadEland(data):
	"A generator that outputs (chr, start, end, strand) tuples for each line in \
	an eland extended formatted iterable."
	for line in data:  #chr12.fa:20086216F10T3A21
		results = re.match(r".*\t(\w+)\t[\d:]+\t(chr\w+)\.fa:(\d+)([F|R]).*$", line)
		if results is None:
			print("Yo file contains ill formatted jibba jabba")
			continue
		seq = results.group(1)
		chrom = results.group(2)
		start = int(results.group(3))
		end = start + len(seq) - 1
		if results.group(4) == 'F':
			strand = '+'
		else:
			strand = '-'
		yield (chrom, start, end, strand)
		
def keyByIndex(inputData, index=0):
	dictionary = {}
	for row in inputData:
		if row[index] in dictionary:
			dictionary[row[index]].append(row)
		else:
			dictionary[row[index]] = [row]
	return dictionary

def sortDictValues(dictionary, key=None, cmp=None):
	"This is an in-place sort of the values of a dictionary of sequences"
	for value in dictionary.values():
		value.sort(key=key, cmp=cmp)

def calculateBinEnrichment(data, binSize):
	"Returns a dictionary of bin enrichment levels for the data indexed in \
	another dictionary by chromosome."
	chromosomes = {}
	totalReads = 0
	numBins = 0
	for row in data:
		chromosome = row[0]
		location = int((row[1]+row[2])/2/binSize)
		if chromosome not in chromosomes:
			chromosomes[chromosome] = {}
		if location not in chromosomes[chromosome]:
			chromosomes[chromosome][location] = 1
			numBins += 1
			totalReads += 1
		else:
			chromosomes[chromosome][location] += 1
			totalReads += 1
	return (chromosomes, (totalReads/numBins))

def formatVariableWiggle(chromosome, span, data):
	wigHeader = """track type=wiggle_0 visibility=full autoScale=on
variableStep  chrom=%s  span=%d
%s"""
	lines = ["%d\t%d" % row for row in data]
	output = '\n'.join(lines)
	return wigHeader % (chromosome, span, output)

def midPointOverlap(one, two, gap=0):
	"A comparator to determine if two peaks' midpoints are overlapping by gap or less." 
	mid1, mid2 = sum(one)/len(one), sum(two)/len(two)
	if  gap < mid2 - mid1:
		return -1
	elif gap < mid1 - mid2: 
		return 1
	else:
		return 0

def midPointNonOverlap(one, two, gap=0):
	"A comparator to determine if two peaks' midpoints are nonoverlapping by gap or less." 
	mid1, mid2 = sum(one)/len(one), sum(two)/len(two)
	if  gap < mid2 - mid1:
		return -1
	elif gap < mid1 - mid2: 
		return 0
	else:
		return 1

def getOverlaps(under, over, comparator=midPointOverlap):
	"takes two dictionaries of sorted lists of peaks keyed by chromosome and returns \
	their overlapping peaks, as determined by comparator."
	for key, peaks1, peaks2 in dictZip(under, over):
		if peaks1 and peaks2:
			for peak in overlap(peaks1, peaks2, comparator):
				yield peak
				
def overlap(one, two, comparator=midPointOverlap):
	"overlap requires one and two to each be a sorted list of pairs of coordinates \
comparable by comparator"
	c1 = c2 = 0
	while c1 < len(one) and c2 < len(two):
		comparison = comparator(one[c1], two[c2])
		if  comparison < 0:
			c1 += 1
		else:  
			if comparison == 0:
				yield two[c2]
			c2 += 1
			
