#!/usr/bin/perl
use strict;
use File::Path qw/make_path/;
use Getopt::Long;
use FindBin qw($Bin); 
use lib "$Bin"; 
use File::Basename;
use File::Spec;

require verifyPath;
require convertFormat;
require peakIntoGeneRef;
require pickHighestScore;
require classifyReads;

if(($ARGV[0] eq "") or ($ARGV[1] eq "") ){die   "-help     Use -help option to display readme file of lonut";
}


#setup my defaults
my $g = 'hg19';
my ($help,$pre,$umCallPeak,$all,$hs,$rType,$qType) = ( 0 ) x 7;
my $pythonBinary ="python";
my $beltBinary ="$Bin/BELT1.0.1_linux64";
my $w = 150;
my $inType = ();
my @pList = 95;
#setup my options
GetOptions(
	'help!' => \$help,
	'py=s'  => \$pythonBinary,
	'belt=s'=> \$beltBinary,
	'r!'	=> \$rType,
	'q!'	=> \$qType,	
	'g=s'   => \$g,
	'pre!'  => \$pre,
	'upeak!'=> \$umCallPeak,
	'w=i'	=> \$w,
	'p=i'	=> \@pList,
	'hs!'	=> \$hs,
	'all!'	=> \$all,
	) or die "Incorrect usage. Use -help option to display readme file of LONUT\n";

if ($help){
       system "cat $Bin/ReadMe";
       exit 0;
}
$beltBinary = verifyPath->inPath($beltBinary);#absolute path of BELT peak calling program


open(IN,$ARGV[0]);
my $line=<IN>;chomp $line;
close(IN);
my @row=split(/\s+/,$line);
my $readLen=length($row[4]);

my $inFull = verifyPath->inPath($ARGV[0]);
my($inBase, $inPath, $inSuffix) = fileparse($inFull, qr/\.[^.]*/);

my $outPath = verifyPath->outPath($ARGV[1], "$inBase");
my $outPrefix = "$outPath/$inBase"; 

my @tmpList = ();my %fileList = ();

my $file_um = "$outPrefix.um";
my $file_mm = "$outPrefix.mm";
my $file_um_bed = "$outPrefix.um.bed";
my $file_mm_bed = "$outPrefix.mm.bed";

if ($pre || $all) {
	unlink <$outPrefix*_*>;
	$inType = "q" if ($qType);
	$inType = "r" if ($rType);
	die "Specify either -r or -q for the bowtie file type\n" if (($qType) && ($rType));
	die "Type of input file should be specified\n" unless ($inType);
	#print "preprocessing\n";
	my ($file_mm,$file_um) = classifyReads->main( $inFull,$inType,$outPrefix );	

	convertFormat->bowtieToBed($file_um,$readLen,"u");
	unlink $file_um;

	convertFormat->bowtieToBed($file_mm,$readLen,"m");
	unlink $file_mm;
	#convertFormat->bedToGff($file_um_bed,"$inBase\tUMT","$outPrefix.um.gff");	
}
print "raw reads split into unique mapped and multiple mapped reads\n";
my $beltCommand="$beltBinary -n 0 1 2 4 -g $g -w $w ";
my $beltWP = ();my $p=();
foreach $p (@pList){
	$p = $p/100; 
	$beltWP = "_W$w"."_P$p";	
	umPeak($p);
}
print "unique-matched reads: peak called by BELT\n";
print "multiple-matched reads: combine with unique-matched reads\n";
foreach $p (@pList){
	$beltWP = "_W$w"."_P$p";
	verifyPath->outPath($outPath,"p$p");
	highScoreMM($p);
	cmPeak($p);
	print "combined reads at p-value = $p is in " . $fileList{$p}{"cmBed"}."\n";
}
print "unique-matched reads in $file_um_bed\n";

sub highScoreMM{	
	my $p = $_[0];
	$fileList{$p}{"hs"} = "$outPath/p$p/$inBase.mm.bed$beltWP.VsPeak.HS.bed";
	$fileList{$p}{"mmVsPeak"}= "$outPath/p$p/$inBase.mm.bed$beltWP.VsPeak";	
	if ($hs || $all){
		my $command = join(" ","$pythonBinary $Bin/findRegions.py -a",
				  $fileList{$p}{"umRef"},
				  $file_mm_bed,
				  $fileList{$p}{"mmVsPeak"});
		#print "multiple mapped reads: findRegions.py\t";
		system($command);
		#print "finished\n";
		pickHighestScore->main($fileList{$p}{"mmVsPeak"},$w,$p);
	}	
}


sub cmPeak{
	my $p = $_[0];
	$fileList{$p}{"cmBed"}="$outPath/p$p/$inBase.cm$beltWP.bed";
	unlink $fileList{$p}{"cmBed"};
	my $command = join(" ","cat", 
			   $file_um_bed,
			   $fileList{$p}{"hs"},
			   ">",$fileList{$p}{"cmBed"});
	system($command);

}

sub umPeak{
	my $p = $_[0];
	$fileList{$p}{"umPeak"} = "$file_um_bed$beltWP.bed";#belt output file
	$fileList{$p}{"umRef"}	= $fileList{$p}{"umPeak"} . ".GeneRef";
	if ($umCallPeak || $all ) {
		unlink <$file_um_bed$beltWP*>;	
		my $command = "$beltCommand -p $p -f $file_um_bed";
		#print "belt:$command\t";	
		system($command);
		#print "finished\n";
		peakIntoGeneRef->main($fileList{$p}{"umPeak"});				
	}
}

